library ieee;
use ieee.std_logic_1164.all;
use work.pack_2.all;

entity uut is
  port (x_fghij : in  X01_vector(20 downto 13);
        z_abcde : out X01_vector(20 downto 13));
end uut;

architecture structure of uut is
  component sn7400 -- 2er-nand
    port (x : in  X01_vector (1 to 2);
          y : out X01);
  end component;
  component sn7427 is -- 3er-nor
    port (x : in X01_vector (1 to 3); y : out X01);
  end component;
  component sn74260 is -- 5er-nor
    port (x : in X01_vector (1 to 5); y : out X01);
  end component;
  component sn7410 -- 3er-nand
    port (x : in  X01_vector (1 to 3);
          y : out X01);
  end component;
  component sn7420 is -- 4er-nand
    port (x : in X01_vector (1 to 4); y : out X01);
  end component;

  signal na, nb, nc, nd : X01; -- not a, not b, not c, not d
  signal s2 : x01_vector(1 to 4); -- nor zwischenergnisse
  signal s3 : x01_vector(1 to 4); -- nand zwischenergnisse

  alias a : x01 is x_fghij(20);
  alias b : x01 is x_fghij(19);
  alias c : x01 is x_fghij(18);
  alias d : x01 is x_fghij(17);
  alias y2: x01 is z_abcde(20); -- nor ergebnis
  alias y3: x01 is z_abcde(19); -- nand ergebnis
begin
  -- block1: a,b,c,d => not a, not b, not c, not d
  u01: sn7400 port map (x(1)=>a, x(2)=>a, y=>na);
  u02: sn7400 port map (x(1)=>b, x(2)=>b, y=>nb);
  u03: sn7400 port map (x(1)=>c, x(2)=>c, y=>nc);
  u04: sn7400 port map (x(1)=>d, x(2)=>d, y=>nd);

  -- block2: 1. nor-ebene
  u10: sn7427 port map (x(1)=>a, x(2)=>b, x(3)=>c, y=>s2(1));
  u11: sn7427 port map (x(1)=>nb, x(2)=>c, x(3)=>nd, y=>s2(2));
  u12: sn7427 port map (x(1)=>b, x(2)=>nc, x(3)=>nd, y=>s2(3));
  u13: sn74260 port map (x(1)=>a, x(2)=>nb, x(3)=>nc, x(4)=>nc, x(5)=>nc, y=>s2(4));

  -- block2: 2. nor-ebene
  u14: sn74260 port map (x(1)=>s2(1), x(2)=>s2(2), x(3)=>s2(3), x(4)=>s2(4), x(5)=>s2(4), y=>y2);

  -- block3: 1. nand-ebene
  u05: sn7410 port map (x(1)=>a, x(2)=>b, x(3)=>c, y=>s3(1));
  u06: sn7410 port map (x(1)=>nb, x(2)=>c, x(3)=>nd, y=>s3(2));
  u07: sn7410 port map (x(1)=>b, x(2)=>nc, x(3)=>nd, y=>s3(3));
  u08: sn7420 port map (x(1)=>a, x(2)=>nb, x(3)=>nc, x(4)=>nc, y=>s3(4));

  -- block3: 2. nand-ebene
  u09: sn7420 port map (x(1)=>s3(1), x(2)=>s3(2), x(3)=>s3(3), x(4)=>s3(4), y=>y3);
end structure;
